package NAT::Dict;

use 5.006;
use strict;
use warnings;
require Exporter;

use NAT;
use MLDBM;
use Fcntl;
use Storable;
use Data::Dumper;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw();

our $VERSION = '0.02';

sub open {
  my $filename = shift;
  my $self = {};
  return undef unless -f $filename;
  my $dic = NAT::dicopen($filename);
  return undef if $dic < 0;
  $self->{id} = $dic;
  return bless $self #amen
}

sub close {
  my $self = shift;
  NAT::dicclose($self->{id});
}

sub add {
  my $self = shift;
  my $other = shift;
  my $dicid = NAT::dicadd($self->{id}, $other->{id});
  return undef if $dicid < 0;
  my $new = { id => $dicid };
  return bless $new #amen
}


sub save {
  my ($dic, $filename) = @_;
  return undef unless NAT::dicsave($dic->{id}, $filename);
  return 1;
}

sub for_each {
  my $self = shift;
  my $sub  = shift;

  my $i = 1;
  while($i < $self->size) {
    &{$sub}( word => $i, occ  => $self->occ($i), vals => $self->vals($i));
    $i++;
  }
}

sub exists {
  my $self = shift;
  my $id = shift;
  return 1 if $id > 0 && $id < $self->size;
  return 0;
}

sub size {
  my $self = shift;
  return NAT::dicgetsize($self->{id});
}

sub occ {
  my $self = shift;
  my $word = shift;
  return NAT::dicgetocc($self->{id}, $word);
}

sub vals {
  my $self = shift;
  my $word = shift;
  return NAT::dicgetvals($self->{id}, $word);
}

1;
__END__

=head1 NAME

NAT::Dict - Perl extension to encapsulate Dict interface

=head1 SYNOPSIS

  use NAT::Dict;

  $corpus = NAT::Dict::open("file.bin");

=head1 DESCRIPTION



=head1 SEE ALSO

See perl(1) and NATools documentation.

=head1 AUTHOR

Alberto Manuel Brandao Simoes, E<lt>albie@alfarrabio.di.uminho.ptE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2003 by NATURA Project
http://natura.di.uminho.pt

This library is free software; you can redistribute it and/or modify
it under the GNU General Public License 2, which you should find on
parent directory. Distribution of this module should be done including
all NATools package, with respective copyright notice.

=cut
