package NAT::Lexicon;

use 5.006;
use strict;
use warnings;
require Exporter;

use NAT;
use MLDBM;
use Fcntl;
use Storable;
use Data::Dumper;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw();

our $VERSION = '0.01';

sub open {
  my $filename = shift;
  return undef unless -f $filename;

  my $wlid = NAT::wlopen($filename);
  return undef if $wlid < 0;

  return bless +{ id => $wlid } #amen
}

sub close {
  my $self = shift;
  NAT::wlclose($self->{id});
}

sub word_from_id {
  my $self = shift;
  my $id = shift;
  return NAT::wlgetbyid($self->{id}, $id);
}

sub id_from_word {
  my $self = shift;
  my $word = shift;
  return NAT::wlgetbyword($self->{id}, lc($word));
}

sub sentence_to_ids {
  my $self = shift;
  my $sentence = shift;
  my @words = split /\s+/, $sentence;
  @words = map { $self->id_from_word($_) } @words;
  return \@words;
}

# We will need something more to handle correct cases
sub ids_to_sentence {
  my $self = shift;
  return join(" ",map { $self->word_from_id($_) } @_);
}

1;
__END__

=head1 NAME

NAT::Lexicon - Perl extension to encapsulate Lexicon files

=head1 SYNOPSIS

  use NAT::Lexicon;

  $lex = NAT::Lexicon::open("file.lex");

  $word = $lex->word_id(2);

  $lex->close;

=head1 DESCRIPTION

...

=head1 SEE ALSO

See perl(1) and NATools documentation.

=head1 AUTHOR

Alberto Manuel Brandao Simoes, E<lt>albie@alfarrabio.di.uminho.ptE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2002-2003 by NATURA Project
http://natura.di.uminho.pt

This library is free software; you can redistribute it and/or modify
it under the GNU General Public License 2, which you should find on
parent directory. Distribution of this module should be done including
all NATools package, with respective copyright notice.

=cut
