#!/usr/bin/perl -w

use CGI qw/:standard/;
use locale;
use NAT;
use NAT::PCorpus;
use Data::Dumper;

my $pcorpus;

print header;

unless (param('path')) {
  print "<tt>path</tt> parameter not specified.\n";
  print ask_for_path("You can enter one here:");
  exit;
}

my $path = param('path');
my $nat = NAT::nat_open($path);
unless ($nat) {
  print "Error opening nat configuration file (<tt>path = '$path'</tt>)\n";
  print ask_for_path("You can enter another here:");
  exit;
}

my $desc = $nat->get_description;

my $rank_is_possible = 0;
$rank_is_possible = 1 if defined $nat->{rank};

my $is_to_rank = param("rankit");
my %rank = ();
%rank = (norank => 1) unless $is_to_rank;
my $rank_file = undef;
$rank_file = $nat->{rank} if $is_to_rank;

if ($nat->{'nr-chunks'} &&
    $nat->{'nr-chunks'} >= 2) {
  %rank = (%rank, chunks => $nat->{'nr-chunks'});
}


if (param('crp') && param('crp') eq $nat->{2}) {
  $pcorpus = NAT::PCorpus::new($nat->{'lex2file'},
			       $nat->{'crp2_'},
			       $nat->{'lex1file'},
			       $nat->{'crp1_'},
			       $rank_file,
			       words => 1, %rank);
} else {
  $pcorpus = NAT::PCorpus::new($nat->{'lex1file'},
			       $nat->{'crp1_'},
			       $nat->{'lex2file'},
			       $nat->{'crp2_'},
			       $rank_file,
			       words => 1, %rank);
}

unless ($pcorpus) { print "Error opening corpus\n"; exit }

print <<"EOCSS";
<html><head>
<style>
 input { border: solid 1px #999;
         padding: 2px;  }
 .even { background-color: #eee; border-bottom: dotted 1px #000;}
 .odd  { background-color: #fff; border-bottom: dotted 1px #000;}
 .form { background-color: #efe;
         margin-bottom: 10px;
         margin-left: 10%;
         margin-right: 10%;
         border: solid 1px #000;
         text-align: center;
       }
 hr { border: none; border-top: solid 1px #000 }
 input[type=submit]:hover {
   background-color: #fff;
         border: solid 1px #000;
         padding: 2px;  }
.header { padding: 3px; border: solid 1px #000; background-color: #def; margin: 2px; }
</style><title>nat-search</title></head><body>
EOCSS

print "<div class=\"form\">\n";
print br,start_form();

print "<table width=\"80%\" align='center'><tr><td>\n";
print "<b>$path</b>: $desc\n",hr;
print "</td></tr><tr><td>\n";
print "Change path:&nbsp;",input({-name=>'path',
				  -value=>$path});
print "</td></tr><tr><td>\n";
my $crp = param('crp') || $nat->{1};
print "Search ", textfield(-name=>'search',-size=>15), " in ",
  popup_menu(-name=>'crp',-values=>[$nat->{1},$nat->{2}],$crp), " co-occurring with ",
  textfield(-name=>'translation',-size=>15);
print "</td></tr><tr><td>\n";
print checkbox(-name=>'rankit',-checked=>1,-label=>"Show translation quality information"),
  "</td></tr><tr><td>\n" if $rank_is_possible;
print submit('Go!');
print "</td></tr></table>\n";
print end_form();
print "</div>\n";

if (param('search')) {
  my $w = param('search');
  my $trans = undef;
  $trans = param('translation') if param('translation');
  my @ans;
  if ($is_to_rank) {
    @ans = $pcorpus->qsearch($w);
  } else {
    @ans = $pcorpus->search($w);
  }
  my $entries = scalar(@ans);

  print "<div class='header'>\n";
  print "Searching '<b>$w</b>'\n";
  print " co-occurring with '<b>$trans</b>'" if ($trans);
  print "</div>\n";
  print "<table>\n";
  $i = 0;
  for (@ans) {
    $i ++;

    if ($is_to_rank) {
      $style0 = " width=\"10\%\" class=\"".($i%2?"even":"odd")."\"";
      $style = " width=\"45\%\" class=\"".($i%2?"even":"odd")."\"";
      $style1 = " width=\"45\%\" class=\"".($i%2?"even":"odd")."\"";
    } else {
      $style = " width=\"50\%\" class=\"".($i%2?"even":"odd")."\"";
      $style1 = " width=\"50\%\" class=\"".($i%2?"even":"odd")."\"";
    }

    print "<tr>";
    if ($is_to_rank) {
      my $rank = shift @$_;
      print "<td$style0>$rank</td>\n";
    }

    $_->[0] =~ s!(\s|^)(${w})(\s|$)!$1<b>$2</b>$3!gi;
    if ($trans) {
      if ($_->[1] =~ m!\b$trans\b!i) {
	$_->[1] =~ s!(\s|^)(${trans})(\s|$)!$1<b>$2</b>$3!gi;
	print "<td$style1>$_->[0]</td><td$style>$_->[1]</td>\n";
      } else {
	$i --;
      }
    } else {
      print "<td$style1>$_->[0]</td><td$style>$_->[1]</td>\n";
    }
    print "</tr>\n";
  }
  print "</table>\n";
  print "<div class='header'>\n";
  print "$i entries found from $entries matching '<b>$w</b>'\n";
  print "</div>\n";

}

sub ask_for_path {
  my $text = shift;
  my $ret = "<p>".start_form();
  $ret .= "$text&nbsp;".input( { -name=>"path" });
  $ret .= input( {-type=>"submit",-value=>"Enter"});
  $ret .= end_form();
}
