/* -*- Mode: C; c-file-style: "stroustrup" -*- */

/**********************************************
 * SEE COPYING FILE FOR COPYRIGHT INFORMATION *
 **********************************************
 * 
 * - dict.c	: library file containing methods to handle dictionary
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include "dict.h"

#define MAXVAL 130  /* maximum value in one cell */
#define MAXDEC 500  /* number of significant decimals: 100 means 2 */
#define MEMBLOCK 1

int AllocMatrix2(struct cMat2 *Matrix, guint32 Nrow, guint32 Ncolumn)
{
    Matrix->Nrows = Nrow;
    Matrix->Ncolumns = Ncolumn;
    if ((Matrix->firstR = (struct cItem **) malloc(Nrow * sizeof(struct cItem *))) == NULL)
	return 1;
    if ((Matrix->firstC = (struct cItem **) malloc(Ncolumn * sizeof(struct cItem *))) == NULL)
	return 2;
    if ((Matrix->items = (struct cItem *) malloc(MEMBLOCK * sizeof(struct cItem))) == NULL)
	return 3;
    Matrix->clean = 0;
    Matrix->p = 0;
    Matrix->memory = MEMBLOCK;
    return 0;
}

void FreeMatrix2(struct cMat2 *Matrix)
{
    free(Matrix->items);
    free(Matrix->firstR);
    free(Matrix->firstC);
}

int IncPointer(struct cMat2 *Matrix)
{
    Matrix->p++;
    if (Matrix->p >= Matrix->memory) {
	Matrix->memory += MEMBLOCK;
	Matrix->items = (struct cItem *) realloc(Matrix->items, (Matrix->memory)*sizeof(struct cItem)); 
    }
    return (Matrix->items == NULL);
}

int DirtyPutValue2(struct cMat2 *Matrix, float f, guint32 r, guint32 c)
{
    struct cItem *I;
    I = Matrix->items + Matrix->p;
    I->row = r; I->column = c;
    I->nextC = NULL;
    if (f < MAXVAL) {
	I->value = (guint32) (f * MAXDEC + 0.5f);
	return IncPointer(Matrix);
    }
    else { 
	I->value = (guint32) (((long) (f* (float) MAXDEC + 0.5f)) % ((long) MAXVAL* (long) MAXDEC)); 
	if (IncPointer(Matrix)) return 1;
	I = Matrix->items + Matrix->p;
	I->row = r; I->column = c;
	I->nextC = NULL;
	I->value = (guint32) (f / (float) MAXVAL);
	return IncPointer(Matrix);      
    }
}

void MarkDoubleItems(struct cMat2 *Matrix)
{
    long i;
    struct cItem *I;
    I = Matrix->items;
    i = 0;
    while (i < Matrix->p - 1) {
	if (I[i].row == I[i+1].row && I[i].column == I[i+1].column)
	    I[i].nextC = I+i+1;
	else
	    I[i].nextC = NULL;
	i++;
    }
}


void SwapItems(struct cMat2 *Matrix, long i, long j)
{
    long l;
    struct cItem dummy, *I;
    I = Matrix->items;
    if (i>j) { l=i; i=j; j=l; }
    dummy = I[i];
    I[i] = I[j];
    I[j] = dummy;
    if (i < Matrix->p -1 && I[i].nextC == I+i+1) {
	if (j < Matrix->p -1 && I[j].nextC == I+j+1) {
	    dummy = I[i+1];
	    I[i+1] = I[j+1];
	    I[j+1] = dummy;
	}
	else {
	    dummy = I[i+1];
	    memmove(I+i+1, I+i+2, (j-i-1) * sizeof(struct cItem));
	    I[j] = dummy;
	}
    }
    else {
	if (j < Matrix->p -1 && I[j].nextC == I+j+1) {
	    dummy = I[j+1];
	    memmove(I+i+2, I+i+1, (j-i) * sizeof(struct cItem));
	    I[i+1] = dummy;
	}
    }  
}

void RemoveItem(struct cMat2 *Matrix, long i)
{
    struct cItem *I;
    I = Matrix->items;
    if (I[i].nextC == I+i+1) 
	memmove(I+i, I+i+2, (Matrix->p -i -2) * sizeof(struct cItem));
    else
	memmove(I+i, I+i+1, (Matrix->p -i -1) * sizeof(struct cItem));
}

void BubbleSortItems(struct cMat2 *Matrix)
{
    long i, j;
    int done;
    struct cItem *I;
    I = Matrix->items;
    i = 0;
    done = 1;
    do {
	while (i < Matrix->p -1) {
	    j = i+1;
	    if (i < Matrix->p -2 && I[i].nextC == I+i+1) j++;
	    if (I[i].row > I[j].row) {
		SwapItems(Matrix, j, i);
		done = 0;
	    } 
	    else {
		if (I[i].row == I[j].row && I[i].column > I[j].column) {
		    SwapItems(Matrix, j, i);
		    done = 0;      
		}
		else {
		    if (I[i].row == I[j].row && I[i].column == I[j].column)
			RemoveItem(Matrix, i);
		}
	    }
	    i = j; 
	}
    } while (!done);
}

void SetRowPointers(struct cMat2 *Matrix)
{
    long l;
    guint32 i;
    l=0;
    for (i=1; i<= Matrix->Nrows; i++) {
	while (l < Matrix->p && Matrix->items[l].row < i) l++;
	if (Matrix->items[l].row == i)
	    Matrix->firstR[i-1] = Matrix->items + l;
	else
	    Matrix->firstR[i-1] = NULL;
    }
}  

void SetColumnPointers(struct cMat2 *Matrix)
{
    long l;
    guint32 i;
    struct cItem *I;
    for (i=1; i <= Matrix->Ncolumns; i++)
	Matrix->firstC[i-1] = NULL;
    l=0;
    while (l < Matrix->p) {
	Matrix->items[l].nextC = NULL;
	I = Matrix->firstC[Matrix->items[l].column - 1];
	if (I == NULL) { 
	    Matrix->firstC[Matrix->items[l].column - 1] = Matrix->items + l;
	}
	else {
	    while (I->nextC != NULL) 
		I = I->nextC;
	    I->nextC = Matrix->items + l;
	}
	l++;
    } 
}

int CleanMatrix2(struct cMat2 *Matrix)
{
    if (Matrix->p <= 0) return 1;
    MarkDoubleItems(Matrix);
    BubbleSortItems(Matrix);
    SetRowPointers(Matrix);
    SetColumnPointers(Matrix);
    Matrix->clean = 1;
    Matrix->items[Matrix->p].row = 0;  /* voor GetValue */ 
    return 0;
}

float GetValue2(struct cMat2 *Matrix, guint32 r, guint32 c)
{
    long i;
    float f;
    struct cItem *p;

    if (r < 1 || r > Matrix->Nrows || c < 1 || c > Matrix->Ncolumns)
	return 0.0f;
    if (!Matrix->clean) 
	if (CleanMatrix2(Matrix)) return 0.0f;

    p = Matrix->firstR[r-1];
    if (p == NULL) return 0.0f;
    i = 0;
    while (p[i].row == r && p[i].column >0 && p[i].column < c) i++;
    if (p[i].row == r && p[i].column == c) {
	f = (float) (p[i].value) / (float) MAXDEC;
	if (p[i+1].row == r  && p[i+1].column == c)
	    f += (float) (p[i+1].value) * MAXVAL;
	return f;
    }
    else 
	return 0.0f;     
}

float GetRowMax2(struct cMat2 *Matrix, guint32 r, guint32 *c, float *f, guint32 max)
{
    long i, j, k;
    float fm, total;
    struct cItem *p;

    if (r < 1 || r > Matrix->Nrows)
	return 0.0f;
    if (!Matrix->clean) 
	if (CleanMatrix2(Matrix)) return 0.0f; /* eigenlijk fout */

    for (j = 0; j < max; j++) {
	f[j] = 0.0f;
	c[j] = 0;
    }
    p = Matrix->firstR[r-1];
    if (p == NULL) return 0.0f;
    i = 0;
    total = 0.0f;
    while (p[i].row == r) {
	fm = (float) (p[i].value) / (float) MAXDEC;
	if (p[i+1].row == r  && p[i].column == p[i+1].column) {
	    fm += (float) (p[i+1].value) * MAXVAL;
	    i++;
	}

	total += fm;
	j = 0;
	while (j < max && c[j] > 0 && f[j] >= fm) j++;
	if (j<max) {
	    for(k = max-1; k >j; k--) {
		c[k] = c[k-1];
		f[k] = f[k-1];
	    }
	    c[j] = p[i].column;
	    f[j] = fm;
	}
	i++;    
    }
    return total;
}


float GetColumnMax2(struct cMat2 *Matrix, guint32 c, guint32 *r, float *f, guint32 max)
{
    long j, k;
    float fm, total;
    struct cItem *p;

    if (c < 1 || c > Matrix->Ncolumns)
	return 0.0f;
    if (!Matrix->clean) 
	if (CleanMatrix2(Matrix)) return 0.0f;  /* eigenlijk fout */

    for (j = 0; j < max; j++) {
	f[j] = 0.0f;
	r[j] = 0;
    }
    total = 0;
    p = Matrix->firstC[c-1];
    while (p != NULL) {
	fm = (float) (p[0].value) / (float) MAXDEC;
	if (p[1].column == c  && p[0].row == p[1].row) {
	    fm += (float) (p[1].value) * MAXVAL;
	    p++;
	}

	total += fm;
	j = 0;
	while (j < max && r[j] > 0 && f[j] >= fm) j++;
	if (j<max) {
	    for(k = max-1; k >j; k--) {
		r[k] = r[k-1];
		f[k] = f[k-1];
	    }
	    r[j] = p[0].row;
	    f[j] = fm;
	}
	p = p->nextC;    
    }
    return total;
}

int LoadMatrix2(struct cMat2 *Matrix, const char *filename)
{
    FILE *fd;
    long l;

    fd = fopen(filename, "rb");
    if (fd == NULL) return 1;
    if (fread(&Matrix->Nrows, sizeof(guint32), 1, fd) != 1) return 2;
    if (fread(&Matrix->Ncolumns, sizeof(guint32), 1, fd) != 1) return 3;
    if (fread(&Matrix->p, sizeof(long), 1, fd) != 1) return 4;
    Matrix->memory = Matrix->p;
    Matrix->items = (struct cItem *) malloc((Matrix->memory + 1) * sizeof(struct cItem));
    Matrix->firstR = (struct cItem **) malloc(Matrix->Nrows * sizeof(struct cItem *));
    Matrix->firstC = (struct cItem **) malloc(Matrix->Ncolumns * sizeof(struct cItem *));
    if (Matrix->items == NULL || Matrix->firstR == NULL || Matrix->firstC == NULL) return 5;
    for(l=0; l<Matrix->memory; l++)
	if (fread(&(Matrix->items[l].row), 3 * sizeof(guint32), 1, fd) != 1) return 6;
    SetRowPointers(Matrix);
    SetColumnPointers(Matrix);
    Matrix->clean = 1;
    Matrix->items[Matrix->p].row = 0;  /* voor GetValue */ 
    fclose(fd);
    return 0;
}

int SaveMatrix2(struct cMat2 *Matrix, const char *filename)
/* Matrix must be clean! */
{
    FILE *fd;
    long l;
    guint32 i;

    if (!Matrix->clean)
	if (CleanMatrix2(Matrix)) return 1;
    fd = fopen(filename, "wb");
    if (fd == NULL) return 2;
    i = Matrix->Nrows;
    if (fwrite(&i, sizeof(i), 1, fd) != 1) return 3;
    i = Matrix->Ncolumns;
    if (fwrite(&i, sizeof(i), 1, fd) != 1) return 4;
    l = Matrix->p;
    if (fwrite(&l, sizeof(l), 1, fd) != 1) return 5;
    for (l=0; l < Matrix->p; l++)
	if (fwrite(&(Matrix->items[l].row), 3 * sizeof(guint32), 1, fd) != 1) return 6;
    if (fclose(fd)) return 7;
    return 0;
}
