/* -*- Mode: C; c-file-style: "stroustrup" -*- */

/**********************************************
 * SEE COPYING FILE FOR COPYRIGHT INFORMATION *
 **********************************************/

#include "standard.h"

#define GetNRow(x)      (x->Nrows)
#define GetNColumn(x)   (x->Ncolumns)

typedef enum cMatrixVal {
    MATRIX_2 = 0,
    MATRIX_1 = 1
} MatrixVal;

typedef struct cCell {
    guint32 column;
    guint32 value1;
    guint32 value2;
} Cell;

typedef struct cRow {
    guint32 length;
    Cell *cells;
} Row;

typedef struct cMatrix {
    guint32 Nrows, Ncolumns;
    Row *rows;
} Matrix;

Matrix* AllocMatrix(guint32 Nrow, guint32 Ncolumn);
void     FreeMatrix(Matrix *matrix);

int PutValue(Matrix *matrix, MatrixVal Ma, float f,
	     guint32 r, guint32 c);

int IncValue(Matrix *matrix, MatrixVal Ma, float incf,
	     guint32 r, guint32 c);

float GetValue(Matrix *matrix, MatrixVal Ma, 
	       guint32 r, guint32 c);

Matrix* LoadMatrix(char *filename);
int SaveMatrix(Matrix *matrix, char *filename);

void MatrixEntropy(Matrix *matrix, MatrixVal Ma, 
		   double *h, double *hx, double *hy, 
		   double *hygx, double *hxgy, double *uygx, double *uxgy, double *uxy);

float GetRow(Matrix *matrix, MatrixVal Ma, 
	     guint32 r, guint32 *c, float *f);

float GetRowMax(Matrix *matrix, MatrixVal Ma, 
		guint32 r, guint32 *c, float *f, guint32 max);
float GetColumnMax(Matrix *matrix, MatrixVal Ma, 
		   guint32 c, guint32 *r, float *f, guint32 max);
int GetPartialMatrix(Matrix *matrix, MatrixVal Ma, 
		     guint32 *r, guint32 *c, double *M, guint32 max);
int GetConditionalMatrix(Matrix *matrix, MatrixVal Ma, 
			 guint32 *r, guint32 *c, float *M, guint32 max);
void ClearMatrix(Matrix *matrix, MatrixVal Ma);
void CopyMatrix(Matrix *matrix,  MatrixVal Mdest);
float CompareMatrices(Matrix *matrix);

/* Gives mean difference */
float MatrixTotal(Matrix *matrix, MatrixVal Ma);
void ColumnTotals(Matrix *matrix, MatrixVal Ma, float *cf);
unsigned long BytesInUse(Matrix *matrix);
